@@ -22,13 +22,14 @@ from sqlalchemy.sql.elements import BinaryExpression
 
 from . import model
 from .column_types import WriteOnlyColumn
+from .serializers import STOP_VALUE
 from .utils import xlate
 from .doc_utils import (
     process_expression,
     _property,
     _function,
     _binary_expression,
-    _foreign_keys
+    _foreign_keys,
 )
 
 
@@ -54,8 +55,14 @@ class DB(object):
     """
 
     def __init__(
-            self, config, model_class=None, checkfirst=True, echo=False,
-            *args, **kwargs):
+        self,
+        config,
+        model_class=None,
+        checkfirst=True,
+        echo=False,
+        *args,
+        **kwargs
+    ):
 
         # not a fan of this
         if config != ":memory:":
@@ -67,7 +74,7 @@ class DB(object):
             self.__setattr__(key, sqlalchemy.__dict__.__getitem__(key))
 
         # column types
-        self.__setattr__('WriteOnlyColumn', WriteOnlyColumn)
+        self.__setattr__("WriteOnlyColumn", WriteOnlyColumn)
 
         # these are being added on an as-needed basis
         orm_functions = ["relationship", "aliased", "lazyload"]
@@ -82,7 +89,8 @@ class DB(object):
 
         # now create_session is done after importing models
         self.session = self.create_session(
-            checkfirst=checkfirst, echo=echo, *args, **kwargs)
+            checkfirst=checkfirst, echo=echo, *args, **kwargs
+        )
 
     @staticmethod
     def load_model_class(model_class=None):
@@ -215,9 +223,7 @@ class DB(object):
         cls.query = self.session.query(cls)
         cls.db = self
 
-    def doc_tables(
-        self, class_list=None, to_camel_case=False, column_props=None
-    ):
+    def doc_tables(self, class_list=None, to_camel_case=False):
         """ doc_tables
 
         This function creates a dictionary of all the table configuratons.
@@ -231,16 +237,13 @@ class DB(object):
         Usage:
             doc_tables(
                 class_list=None
-                to_camel_case=False,
-                column_props=None
+                to_camel_case=False
             )
 
         Args:
             #class_list: (None : list) : if left as None, returns all classes
             to_camel_case: (bool) : converts the column names to camel case
             serial_fields: (None : list) : specify a limited list of columns
-            column_props: (None : list) : filter column details to specific
-                                          items
 
         Return:
 
@@ -262,9 +265,7 @@ class DB(object):
         ]
 
         doc_list = [
-            self.doc_table(
-                cls, to_camel_case=to_camel_case, column_props=column_props
-            )
+            self.doc_table(cls, to_camel_case=to_camel_case)
             for cls in classes
             if isinstance(cls, type) and issubclass(cls, self.Model)
         ]
@@ -275,7 +276,12 @@ class DB(object):
         return doc
 
     def doc_table(
-        self, cls, to_camel_case=False, serial_fields=None, column_props=None
+        self,
+        cls,
+        to_camel_case=False,
+        serial_fields=None,
+        serial_field_relations=None,
+        level_limits=None,
     ):
         """ doc_table
 
@@ -293,22 +299,23 @@ class DB(object):
                 cls,
                 to_camel_case=False,
                 serial_fields=None,
-                column_props=None
+                serial_field_relations=None,
+                level_limits=None
             )
 
         Args:
             cls: (class) : the table to be documented
             to_camel_case: (bool) : converts the column names to camel case
             serial_fields: (None : list) : specify a limited list of columns
-            column_props: (None : list) : filter column details to specific
-                                          items
+            serial_field_relations: (None: dict) : Can control what serial
+            fields are included in relationships
+            level_limits: (None : set) : A technical variable related to
+            preventing runaway recursion. Best to leave it alone.
 
         Return:
 
             doc (dict) : a dict of the column values
 
-        The column props included are name, type, required, default detail,
-        foreign_keys, and unique.
         """
 
         def _post_value(key, item_dict):
@@ -317,6 +324,14 @@ class DB(object):
                 key = xlate(key)
             properties[key] = item_dict
 
+        if level_limits is None:
+            level_limits = set()
+
+        if cls._class() in level_limits:
+            # it has already been done
+            if not cls._has_self_ref():
+                return STOP_VALUE
+
         properties = {}
         doc = {
             cls.__name__: {
@@ -345,26 +360,31 @@ class DB(object):
                     # done afterwards because keys in expression can change
 
                 elif isinstance(value.expression, BinaryExpression):
-                    item_dict = _binary_expression(value)
+                    # relationship or None
+                    item_dict = _binary_expression(
+                        value,
+                        to_camel_case=to_camel_case,
+                        serial_field_relations=serial_field_relations,
+                        level_limits=level_limits,
+                        bidirectional=cls._is_bidirectional(key),
+                    )
                 else:
                     item_dict = {
                         "readOnly": True,
                         "unknown": str(value.expression),
                     }
-                _filter_column_props(column_props, item_dict)
-                _post_value(key, item_dict)
+                if item_dict is not None:
+                    _post_value(key, item_dict)
 
             elif isinstance(value, property):
 
                 item_dict = _property(cls, key, value)
-                _filter_column_props(column_props, item_dict)
                 _post_value(key, item_dict)
 
             elif callable(value):
 
                 item_dict = _function(value)
                 if item_dict is not None:
-                    _filter_column_props(column_props, item_dict)
                     _post_value(key, item_dict)
 
             else:
@@ -379,7 +399,7 @@ class DB(object):
 
     def _process_table_args(self, cls):
 
-        table_args = cls.__dict__.get('__table_args__')
+        table_args = cls.__dict__.get("__table_args__")
         key = "constraints"
         constraints = []
 
@@ -389,22 +409,23 @@ class DB(object):
             if isinstance(table_args, tuple):
                 for item in table_args:
                     if isinstance(item, self.CheckConstraint):
-                        ddict = {'check_constraint': item.sqltext.text}
+                        ddict = {"check_constraint": item.sqltext.text}
                         constraints.append(ddict)
 
                     elif isinstance(item, self.ForeignKeyConstraint):
-                        fk_key = 'foreign_key_constraint'
+                        fk_key = "foreign_key_constraint"
                         ddict = {fk_key: {}}
                         ddict[fk_key].update(_foreign_keys(item.elements))
-                        ddict[fk_key].update({'column_keys': item.column_keys})
+                        ddict[fk_key].update({"column_keys": item.column_keys})
 
                         constraints.append(ddict)
 
                     elif isinstance(item, self.UniqueConstraint):
-                        ddict = {'unique_contraint': {'columns': []}}
+                        ddict = {"unique_contraint": {"columns": []}}
                         for uniq_col in item.columns:
-                            ddict['unique_contraint']['columns'].append(
-                                uniq_col.expression.name)
+                            ddict["unique_contraint"]["columns"].append(
+                                uniq_col.expression.name
+                            )
                         constraints.append(ddict)
         if constraints:
             return {key: constraints}
@@ -424,21 +445,3 @@ class DB(object):
             dict
         """
         return process_expression(getattr(cls, column_name).expression)
-
-
-def _filter_column_props(column_props, item_dict):
-    """ _filter_column_props
-
-    This function removes unneeded column properties.
-
-    Args:
-        column_props: (dict) : Only props to return
-        item_dict: (dict) : the dict set to processes
-
-    Returns:
-        dict
-    """
-    if column_props is not None:
-        for prop_key in list(item_dict.keys()):
-            if prop_key not in column_props:
-                del item_dict[prop_key]
